package com.example.assignment1;

import android.os.Bundle;
import android.text.InputFilter;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;

public class SettingsActivity extends AppCompatActivity {

    protected Button saveButton;
    protected Toolbar myToolbar;

    protected int maxLength = 16;

    protected CounterButtonSharedPreferenceHelper counterButtonSharedPrefHelper;

    protected CountSharedPreferenceHelper countSharedPrefHelper;

    protected EditText counter_1_name_edit_text;
    protected EditText counter_2_name_edit_text;
    protected EditText counter_3_name_edit_text;

    protected EditText maximum_counts_edit_text;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_settings);

        myToolbar = (Toolbar) findViewById(R.id.settings_toolbar);
        saveButton = (Button) findViewById(R.id.saveButton);

        counter_1_name_edit_text = (EditText) findViewById(R.id.counter_1_edit_text);
        counter_2_name_edit_text = (EditText) findViewById(R.id.counter_2_edit_text);
        counter_3_name_edit_text = (EditText) findViewById(R.id.counter_3_edit_text);
        maximum_counts_edit_text = (EditText) findViewById(R.id.maximum_counts_edit_text);

        counter_1_name_edit_text.setFilters(new InputFilter[]{new InputFilter.LengthFilter(maxLength)});
        counter_2_name_edit_text.setFilters(new InputFilter[]{new InputFilter.LengthFilter(maxLength)});
        counter_3_name_edit_text.setFilters(new InputFilter[]{new InputFilter.LengthFilter(maxLength)});

        counterButtonSharedPrefHelper = new CounterButtonSharedPreferenceHelper(SettingsActivity.this);
        countSharedPrefHelper = new CountSharedPreferenceHelper(SettingsActivity.this);

        // enable toolbar/actionbar
        setSupportActionBar(myToolbar);

        assert getSupportActionBar() != null;
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);   //show back button

        goToSettingsActivityDisplayMode();
    }

    @Override
    protected void onStart() {
        super.onStart();

        if (counterButtonSharedPrefHelper.getCounterButtonName(CounterButton.FIRST).equals("") || counterButtonSharedPrefHelper.getCounterButtonName(CounterButton.SECOND).equals("") || counterButtonSharedPrefHelper.getCounterButtonName(CounterButton.THIRD).equals("")) {

            goToSettingsActivityEditMode();

        } else {
            counter_1_name_edit_text.setHint(counterButtonSharedPrefHelper.getCounterButtonName(CounterButton.FIRST));
            counter_2_name_edit_text.setHint(counterButtonSharedPrefHelper.getCounterButtonName(CounterButton.SECOND));
            counter_3_name_edit_text.setHint(counterButtonSharedPrefHelper.getCounterButtonName(CounterButton.THIRD));
            maximum_counts_edit_text.setHint(String.format(Integer.toString(countSharedPrefHelper.getCurrentCount())));
        }

    }

    // handles edit option
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();

        if (id == R.id.edit_settings) {
            goToSettingsActivityEditMode();
            return true;
        }

        return super.onOptionsItemSelected(item);
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.settings_activity_menu, menu);
        return true;
    }

    protected void goToSettingsActivityDisplayMode() {
        // no save button in Display Mode
        saveButton.setVisibility(View.INVISIBLE);

        // in DISPLAY mode, it's not possible to edit counter button names
        counter_1_name_edit_text.setEnabled(false);
        counter_2_name_edit_text.setEnabled(false);
        counter_3_name_edit_text.setEnabled(false);
        maximum_counts_edit_text.setEnabled(false);
    }

    protected void goToSettingsActivityEditMode() {

        saveButton.setVisibility(View.VISIBLE);

        counter_1_name_edit_text.setEnabled(true);
        counter_2_name_edit_text.setEnabled(true);
        counter_3_name_edit_text.setEnabled(true);
        maximum_counts_edit_text.setEnabled(true);

        counter_1_name_edit_text.setText(counterButtonSharedPrefHelper.getCounterButtonName(CounterButton.FIRST));
        counter_2_name_edit_text.setText(counterButtonSharedPrefHelper.getCounterButtonName(CounterButton.SECOND));
        counter_3_name_edit_text.setText(counterButtonSharedPrefHelper.getCounterButtonName(CounterButton.THIRD));
        maximum_counts_edit_text.setText(String.format(Integer.toString(countSharedPrefHelper.getMaxCount())));

        String curr_text1 = counter_1_name_edit_text.getText().toString();
        String curr_text2 = counter_2_name_edit_text.getText().toString();
        String curr_text3 = counter_3_name_edit_text.getText().toString();
        int curr_maximum_count = Integer.parseInt(maximum_counts_edit_text.getText().toString());

        saveButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                // counter button strings are checked statically in the activity XML file
                if (check_max_count_input()) {
                    // Get the text from EditText fields
                    String text1 = counter_1_name_edit_text.getText().toString();
                    String text2 = counter_2_name_edit_text.getText().toString();
                    String text3 = counter_3_name_edit_text.getText().toString();
                    int maximum_count = Integer.parseInt(maximum_counts_edit_text.getText().toString());

                    counterButtonSharedPrefHelper.saveCounterButtonName(CounterButton.FIRST, text1);
                    counterButtonSharedPrefHelper.saveCounterButtonName(CounterButton.SECOND, text2);
                    counterButtonSharedPrefHelper.saveCounterButtonName(CounterButton.THIRD, text3);
                    countSharedPrefHelper.saveMaxCount(maximum_count);

                    // if the fields been changed, we reset the count and the events list
                    if (
                            !(curr_text1.equals(text1)) ||
                                    !(curr_text2.equals(text2)) ||
                                    !(curr_text3.equals(text3)) ||
                                    !(maximum_count == curr_maximum_count)
                    ){
                        countSharedPrefHelper.setEventsList("");
                    }

                    goToSettingsActivityDisplayMode();
                }
            }
        });
    }

    protected Boolean check_max_count_input() {
        try {
            int countInput = Integer.parseInt(maximum_counts_edit_text.getText().toString());
            if (countInput < 5 || countInput > 200) {
                Toast.makeText(getApplicationContext(), "Number must be between 5 and 200 inclusive", Toast.LENGTH_SHORT).show();
                return false;
            }
        } catch (NumberFormatException e) {
            Toast.makeText(getApplicationContext(), "Number must be between 5 and 200 inclusive", Toast.LENGTH_SHORT).show();
            return false;
        }

        return true;
    }
}